# Contribute to this code sample 

Thank you for your interest in this sample! Your contributions and improvements will help the developer community. 

## Ways to contribute 

There are several ways you can contribute to this sample: providing better code comments, fixing open issues, and adding new features. 

### Provide better code comments 

Code comments make code samples even better by helping developers learn to use the code correctly in their own applications. If you spot a class, method, or section of code that you think could use better descriptions, then create a pull request with your code comments.


In general we want our code comments to follow these guidelines: 

- Any code that has associated documentation displayed in an IDE (such as IntelliSense, or JavaDocs) has code comments.
- Classes, methods, parameters, and return values have clear descriptions.
- Exceptions and errors are documented.
- Remarks exist for anything special or notable about the code.
- Sections of code that have complex algorithms have appropriate comments describing what they do.
- Code added from Stack Overflow, or any other source, is clearly attributed. 

### Fix open issues 

Sometimes we get a lot of issues, and it can be hard to keep up. If you have a solution to an open issue that hasn't been addressed, fix the issue and then submit a pull request. 

### Add a new feature 

New features are great! Be sure to check with the repository admin first to be sure the feature will fit the intent of the sample. Start by opening an issue in the repository that proposes and describes the feature. The repository admin will respond and may ask for more information. If the admin agrees to the new feature, create the feature and submit a pull request. 

## Contribution guidelines 

We have some guidelines to help maintain a healthy repo and code for everyone. 

### The Contribution License Agreement 

For most contributions, you'll be asked to sign a Contribution License Agreement (CLA). This will happen when you submit a pull request. Microsoft will send a link to the CLA to sign via email. Once you sign the CLA, your pull request can proceed. Read the CLA carefully, because you may need to have your employer sign it. 

### Code contribution checklist 

Be sure to satisfy all of the requirements in the following list before submitting a pull request: 

- Follow the code style that is appropriate for the platform and language in this repo. For example, Android code follows the style conventions found in the [Code Style for Contributors guide](https://source.android.com/source/code-style.html).
- Test your code.
- Test the UI thoroughly to be sure nothing has been broken by your change.
- Keep the size of your code change reasonable. if the repository owner cannot review your code change in 4 hours or less, your pull request may not be reviewed and approved quickly.
- Avoid unnecessary changes. The reviewer will check differences between your code and the original code. Whitespace changes are called out along with your code. Be sure your changes will help improve the content. 

### Submit a pull request to the master branch 

When you're finished with your work and are ready to have it merged into the master repository, follow these steps. Note: pull requests are typically reviewed within 10 business days. If your pull request is accepted you will be credited for your submission. 

1. Submit your pull request against the master branch.
2. Sign the CLA, if you haven't already done so.
3. One of the repo admins will process your pull request, including performing a code review. If there are questions, discussions, or change requests in the pull request, be sure to respond.
4. When the repo admins are satisfied, they will accept and merge the pull request. 

Congratulations, you have successfully contributed to the sample! 

## FAQ 

### Where do I get a Contributor's License Agreement? 

You will automatically be sent a notice that you need to sign the Contributor's License Agreement (CLA) if your pull request requires one. 

As a community member, you must sign the CLA before you can contribute large submissions to this project. You only need complete and submit the CLA document once. Carefully review the document. You may be required to have your employer sign the document. 

### What happens with my contributions? 

When you submit your changes, via a pull request, our team will be notified and will review your pull request. You will receive notifications about your pull request from GitHub; you may also be notified by someone from our team if we need more information. We reserve the right to edit your submission for legal, style, clarity, or other issues. 

### Who approves pull requests? 

The admin of the repository approves pull requests. 

### How soon will I get a response about my change request or issue? 

We typically review pull requests and respond to issues within 10 business days. 

## More resources 

- To learn more about Markdown, see [Daring Fireball](http://daringfireball.net/).
- To learn more about using Git and GitHub, check out the [GitHub Help section](http://help.github.com/). 